use ircparser;
use pgp::{composed::signed_key::*, Deserializable};
use std::sync::mpsc;

fn initialize_connection(
    tx: mpsc::Sender<String>,
    rx: mpsc::Receiver<String>,
    person: &str,
    key_string: &str,
) -> Result<SignedPublicKey, pgp::errors::Error> {
    tx.send(format!("PRIVMSG {person} INITIALIZE"))
        .expect("Closed");

    for line in key_string.split('\n') {
        tx.send(format!("PRIVMSG {person} {line}")).unwrap();
    }

    let mut key_lines: Vec<String> = vec![];
    let mut line: String = String::new();

    while line != "END" {
        key_lines.push(line);
        line = match ircparser::parse(&rx.recv().expect("Channel Closed")) {
            Ok(parsed) => {
                let line = &parsed[0];

                line.params[1].clone()
            }
            Err(e) => {
                println!("WARN: got invalid line {e}");
                String::new()
            }
        }
    }

    Ok(SignedPublicKey::from_string(&key_lines.join("\n"))?.0)
}
